! Fortran 90 and PVM
! measure of communication performance
! outer conditions of the experiment (host, dataformat)
!
! Laurent Gasser (lga@sma.ch)
! Swiss Meteorological Institute
! November 25, 1996

module parameters
implicit none

  character(64) :: host_name   = "undefined"
  integer       :: data_set    = -1
  integer       :: data_format = -1
  integer       :: nb_measures = -1

contains

  subroutine Specify_Experiment()
    use metrics
  ! local declarations
    namelist /experiment/ host_name, data_format, data_set, nb_measures, items
    ! cannot use namelist with assumed shape arrays
    logical :: found_namelist
    integer :: items(allowed_measures)
    integer :: i
    character(22), parameter :: alias(-1:2) = (/ &
                                              ", unknown data format ", &
                                              ", alias PVMDATADEFAULT", &
                                              ", alias PVMDATARAW    ", &
                                              ", alias PVMDATAINPLACE" /)
    character(24), parameter :: data_range(0:1) = (/ &
                                              ", rank of item (default)", &
                                              ", from huge down        " /)

  ! body Specify_Experiment

    print *, " Reading the file PingPong.nl for specifications."
    print *
    print *, " The program accepts up to ", allowed_measures, " data packet sizes."
    print *

    inquire(file = "PingPong.nl", exist = found_namelist)
    if (.not. found_namelist) then
       print *, " The namelist file 'PingPong.nl' defining the experiment is not found."
       print *, " Please create one following this model:"
       print *, " &experiment"
       print *, "     host_name = ""e20"","
       print *, "     data_format = 0,"
       print *, "     nb_measures = 9,"
       print *, "     items = 1, 5, 10, 50, 100, 500, 1000, 5000, 10000"
       print *, " /"
       stop
    end if
    open(unit = 10, file = "PingPong.nl", status = "OLD")
    read(unit = 10, nml = experiment)
    close(unit = 10)

    print *, " Specifications of the current experiment:"
    print *
    print *, " host_name [character(64)] = ", TRIM(host_name)
    if ( (data_format < 0) .or. (ubound(alias,1) < data_format) ) then
       print *, " data_format [integer]     = ", data_format, alias(-1)
    else
       print *, " data_format [integer]     = ", data_format, alias(data_format)
    end if
    if ( (data_set < 1) .or. (ubound(data_range, 1) < data_set) ) then
       print *, " data_set [integer]        = ", data_set, data_range(0)
    else
       print *, " data_set [integer]        = ", data_set, data_range(data_set)
    end if
    print *, " nb_measures [integer]     = ", nb_measures
    if ( (nb_measures < 1) .or. (allowed_measures < nb_measures) ) then
       print *, " Ping-Pong cannot handle this number of measures."
       print *, " Abort experiment."
       stop
    end if

    ! transfer specifications in measure array
    do i = lbound(items,1), ubound(items,1)
       print "('  items(', I1, ') [integer]        = ', I5)", i, items(i)
       if (items(i) < 1)  then
          print *, " A data packet must have a positive number of items."
          print *, " Abort experiment."
          stop
       end if
       measure(i)%items = items(i)
    end do
    print *
  end subroutine Specify_Experiment

end module parameters
